@@ -522,7 +522,7 @@ def em_pseudo_data(z_values,
     return theta_t1, k_state, logger
 
 
-def pseudo_likelihood(x_score, threshold=0.0001):
+def pseudo_likelihood(x_score, threshold=0.0001, missing=None):
     """
     pseudo likelhood optimization for the copula model parameters
     :param x_score np.array of score (measures x samples)
