@@ -723,7 +723,7 @@ def merge_beds(bed_files: list,
     if thread_num <= 1:
         merged_files[1:] = list(map(partial(merge_peaks,
                                             peaks=merged_files,
-                                            score_col=score_col ),
+                                            score_col=score_col),
                                     range(len(bed_files) - 1),
                                     ))
     else:
@@ -795,7 +795,8 @@ def process_bed(file_names: list,
                 score_cols: str = None,
                 pos_cols: list = None,
                 drop_unmatched: bool = True,
-                thread_num=mp.cpu_count()
+                thread_num=mp.cpu_count(),
+                missing: float = None
                 ):
     """
     Process a list of bed files names with the first names the merged bed files
@@ -811,6 +812,7 @@ def process_bed(file_names: list,
     :param pos_cols: list of position column name to sort and merge on
     :param drop_unmatched: bool
     :param thread_num: int number of thread to use for merging
+    :param missing:
     :return: nothing
     """
     if file_cols is None:
@@ -831,7 +833,8 @@ def process_bed(file_names: list,
                 np_list=bed_files[1:],
                 score_cols=score_cols
             ),
-            threshold=threshold
+            threshold=threshold,
+            missing=missing
         )
         writefiles(
             bed_files=bed_files,
